[Top][All Lists]
[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]
init.sh: fix EXEEXT shims to work also for names like test-prog
From: |
Jim Meyering |
Subject: |
init.sh: fix EXEEXT shims to work also for names like test-prog |
Date: |
Sun, 21 Feb 2010 17:58:12 +0100 |
Per discussion last week, here's a patch for init.sh.
The main goal is to allow the EXEEXT shim business to work
also for program names like test-program, (containing a hyphen).
I would have liked to skip the invocation of $re_shell_ -c ...
if the current shell is deemed adequate, but determining that is
a rats nest, since it would require use of eval, and Solaris'
/bin/sh handles failing "eval" very poorly. If someone knows how to
make that more efficient (yet still clean and reliable) for common,
working shells like dash-or-bash-as-/bin/sh, I'd welcome the tip.
I suppose it could test e.g., $BASH_VERSION, if bash has supported
the desired features for long enough, or even do version number comparisons,
but the latter seems not worthwhile.
>From 63509688315d9a8c8e60d8e776736b454f3019f5 Mon Sep 17 00:00:00 2001
From: Jim Meyering <address@hidden>
Date: Sun, 21 Feb 2010 17:47:43 +0100
Subject: [PATCH] init.sh: fix EXEEXT shims to work also for names like test-prog
* tests/init.sh: Re-exec a better shell, when needed.
If the current shell lacks support for posix $(...), an init.sh-using
test will now try to find a shell that supports that. If EXEEXT is
nonempty, we also require support for hyphen-in-alias-name and shell
substitutions like ${var#glob}. Failure to find such a shell results
in a skipped test.
---
ChangeLog | 10 ++++++++++
tests/init.sh | 48 +++++++++++++++++++++++++++++++++++++++++-------
2 files changed, 51 insertions(+), 7 deletions(-)
diff --git a/ChangeLog b/ChangeLog
index 3fe0b44..6d35c21 100644
--- a/ChangeLog
+++ b/ChangeLog
@@ -1,3 +1,13 @@
+2010-02-21 Jim Meyering <address@hidden>
+
+ init.sh: fix EXEEXT shims to work also for names like test-prog
+ * tests/init.sh: Re-exec a better shell, when needed.
+ If the current shell lacks support for posix $(...), an init.sh-using
+ test will now try to find a shell that supports that. If EXEEXT is
+ nonempty, we also require support for hyphen-in-alias-name and shell
+ substitutions like ${var#glob}. Failure to find such a shell results
+ in a skipped test.
+
2010-02-21 Bruno Haible <address@hidden>
Really work around around "broken pipe" error message from bash 3.2.
diff --git a/tests/init.sh b/tests/init.sh
index e7664b8..4debc65 100644
--- a/tests/init.sh
+++ b/tests/init.sh
@@ -52,6 +52,41 @@
# 4. Finally
# $ exit
+# We require $(...) support unconditionally.
+# We require a few additional shell features only when $EXEEXT is nonempty,
+# in order to support automatic $EXEEXT emulation:
+# - hyphen-containing alias names
+# - we prefer to use ${var#...} substitution, rather than having
+# to work around lack of support for that feature.
+# The following code attempts to find a shell with support for these features
+# and re-exec's it. If not, it skips the current test.
+
+gl_shell_test_script_='
+test $(echo y) = y || exit 1
+test -z "$EXEEXT" && exit 0
+shopt -s expand_aliases
+alias a-b="echo zoo"
+v=abx
+ test ${v%x} = ab \
+ && test ${v#a} = bx \
+ && test $(a-b) = zoo
+'
+
+if test "x$1" = "x--no-reexec"; then
+ shift
+else
+ for re_shell_ in "${CONFIG_SHELL:-no_shell}" /bin/sh bash dash zsh pdksh fail
+ do
+ test "$re_shell_" = no_shell && continue
+ test "$re_shell_" = fail && skip_ failed to find an adequate shell
+ if "$re_shell_" -c "$gl_shell_test_script_" 2>/dev/null; then
+ exec "$re_shell_" "$0" --no-reexec "$@"
+ echo "$ME_: exec failed" 1>&2
+ exit 127
+ fi
+ done
+fi
+
# We use a trap below for cleanup. This requires us to go through
# hoops to get the right exit status transported through the handler.
# So use `Exit STATUS' instead of `exit STATUS' inside of the tests.
@@ -117,11 +152,11 @@ find_exe_basenames_()
}
# Consider the files in directory, $1.
# For each file name of the form PROG.exe, create a shim function named
# PROG that simply invokes PROG.exe, then return 0. If any selected
# file name or the directory name, $1, contains an unexpected character,
# define no function and return 1.
-create_exe_shim_functions_()
+create_exe_shims_()
{
case $EXEEXT in
'') return 0 ;;
@@ -134,9 +169,7 @@ create_exe_shim_functions_()
if test -n "$base_names_"; then
for base_ in $base_names_; do
- # Create a function named $base whose sole job is to invoke
- # $base_$EXEEXT, assuming its containing dir is already in PATH.
- eval "$base_() { $base_$EXEEXT"' "$@"; }'
+ alias "$base_"="$base_$EXEEXT"
done
fi
@@ -160,8 +193,9 @@ path_prepend_()
esac
PATH="$abs_path_dir_:$PATH"
- # Create a function FOO for each FOO.exe in this directory.
- create_exe_shim_functions_ "$abs_path_dir_"
+ # Create an alias, FOO, for each FOO.exe in this directory.
+ create_exe_shims_ "$abs_path_dir_" \
+ || fail_ "something failed (above): $abs_path_dir_"
shift
done
export PATH
@@ -117,11 +152,11 @@ find_exe_basenames_()
}
# Consider the files in directory, $1.
-# For each file name of the form PROG.exe, create a shim function named
+# For each file name of the form PROG.exe, create an alias named
# PROG that simply invokes PROG.exe, then return 0. If any selected
# file name or the directory name, $1, contains an unexpected character,
--
1.7.0.256.g2d4f4
- init.sh: fix EXEEXT shims to work also for names like test-prog,
Jim Meyering <=